module Fedi.Routes.Inbox where

import Prelude hiding (error)
import Fedi.Types
import Fedi.Helpers
import Fedi.UserDetails
import Web.Twain qualified as Twain
import Web.Twain.Types qualified as Twain
import Control.Monad.Catch (catch, displayException, SomeException)
import Fedi.Signature.Check
import Control.Logger.Simple qualified as Log

-- * Inbox

matchInbox :: UserDetails -> Twain.PathPattern
matchInbox details =
  fromString ("/" <> details.username <> "/inbox")

handleInbox :: (AnyActivity -> Twain.ResponderM Twain.Response) -> Twain.ResponderM b
handleInbox handle = do
  let
    handler = do
      activity <- checkSignatureAndParseBody
      response <- handle activity
      Twain.send response
  handler `catch` \(e :: SomeException) -> do
    Log.logError (pShow $ displayException e)
    Twain.send $
      -- not necessarily 500, but we'll do this for now.
      Twain.status Twain.status500 $
        Twain.text "Internal Server Error 500"
