{-# language RecordWildCards #-}

module Fedi.Signature.Types where

import Prelude hiding (error)
import Fedi.UserDetails
import Data.Text qualified as T
import Data.Text.Encoding qualified as T
import Data.ByteString qualified as BS

data HttpSignature
  = HttpSignature
    { signatureHeader :: SignatureHeader
    , date :: String
    , host :: String
    , digest :: ByteString
    }
  deriving Show

toSignature :: SignatureHeader -> ByteString
toSignature sig =
  BS.intercalate ","
    [ "keyId=\"" <> fromString sig.keyId <> "\""
    , "headers=\"" <> BS.intercalate " " (map T.encodeUtf8 sig.headers) <> "\""
    , "signature=\"" <> sig.signature <> "\""
    , "algorithm=\"" <> "rsa-sha256" <> "\""
    ]

data SignatureHeader
  = SignatureHeader
    { -- | Where to get the public key for this actor
      keyId :: Url
    , -- | Which headers have been sent
      headers :: [T.Text]
    , -- | Contains the signature
      signature :: ByteString
    , components :: [(Component, String)]
    }
  deriving Show

data Component
  = KeyId
  | Headers
  | Signature
  | Algorithm
  | Other String
  deriving (Eq, Show)
